@@ -9,20 +9,19 @@ from .base import Column, library
 
 @library.register
 class TemplateColumn(Column):
-    """
+    '''
     A subclass of `.Column` that renders some template code to use as
     the cell value.
 
-    :type  template_code: `unicode`
-    :param template_code: the template code to render
-    :type  template_name: `unicode`
-    :param template_name: the name of the template to render
+    Arguments:
+        template_code (str): template code to render
+        template_name (str): name of the template to render
 
     A `~django.template.Template` object is created from the
     *template_code* or *template_name* and rendered with a context containing:
 
-    - *record* -- data record for the current row
-    - *value* -- value from `record` that corresponds to the current column
+    - *record*  -- data record for the current row
+    - *value*   -- value from `record` that corresponds to the current column
     - *default* -- appropriate default value to use as fallback
 
     Example:
@@ -35,13 +34,7 @@ class TemplateColumn(Column):
             bar = tables.TemplateColumn(template_name='myapp/name2_column.html')
 
     Both columns will have the same output.
-
-    .. important::
-
-        In order to use template tags or filters that require a
-        `~django.template.RequestContext`, the table **must** be rendered via
-        :ref:`{% render_table %} <template-tags.render_table>`.
-    """
+    '''
     empty_values = ()
 
     def __init__(self, template_code=None, template_name=None, **extra):
@@ -65,9 +58,8 @@ class TemplateColumn(Column):
 
         try:
             if self.template_code:
-                template = Template(self.template_code)
+                return Template(self.template_code).render(context)
             else:
-                template = get_template(self.template_name)
-            return template.render(context)
+                return get_template(self.template_name).render(context.flatten())
         finally:
             context.pop()
