@@ -6,29 +6,34 @@ from itertools import chain
 
 from django.db.models.fields import FieldDoesNotExist
 from django.utils import six
-from django.utils.html import escape
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html_join
 
 
 class Sequence(list):
-    """
-    Represents a column sequence, e.g. ``("first_name", "...", "last_name")``
+    '''
+    Represents a column sequence, e.g. ``('first_name', '...', 'last_name')``
 
     This is used to represent `.Table.Meta.sequence` or the `.Table`
     constructors's *sequence* keyword argument.
 
     The sequence must be a list of column names and is used to specify the
-    order of the columns on a table. Optionally a "..." item can be inserted,
+    order of the columns on a table. Optionally a '...' item can be inserted,
     which is treated as a *catch-all* for column names that aren't explicitly
     specified.
-    """
+    '''
     def expand(self, columns):
-        """
-        Expands the ``"..."`` item in the sequence into the appropriate column
+        '''
+        Expands the ``'...'`` item in the sequence into the appropriate column
         names that should be placed there.
 
-        :raises: `ValueError` if the sequence is invalid for the columns.
-        """
+        arguments:
+            columns (list): list of column names.
+        returns:
+            The current instance.
+
+        raises:
+            `ValueError` if the sequence is invalid for the columns.
+        '''
         ellipses = self.count("...")
         if ellipses > 1:
             raise ValueError("'...' must be used at most once in a sequence.")
@@ -50,74 +55,79 @@ class Sequence(list):
                 columns.pop(columns.index(name))
         self[:] = chain(head, columns, tail)
 
+        return self
+
+
+class OrderBy(str):
+    '''
+    A single item in an `.OrderByTuple` object. This class is essentially just
+    a `str` with some extra properties.
+    '''
+
+    QUERYSET_SEPARATOR = '__'
 
-class OrderBy(six.text_type):
-    """
-    A single item in an `.OrderByTuple` object. This class is
-    essentially just a `str` with some extra properties.
-    """
     @property
     def bare(self):
-        """
-        Return the bare form.
+        '''
+        Returns:
+            `.OrderBy`: the bare form.
 
         The *bare form* is the non-prefixed form. Typically the bare form is
         just the ascending form.
 
         Example: ``age`` is the bare form of ``-age``
 
-        :rtype: `.OrderBy` object
-        """
+        '''
         return OrderBy(self[1:]) if self[:1] == '-' else self
 
     @property
     def opposite(self):
-        """
-        Return an `.OrderBy` object with an opposite sort influence.
+        '''
+        Provides the opposite of the current sorting directon.
 
-        Example:
+        Returns:
+            `.OrderBy`: object with an opposite sort influence.
 
-        .. code-block:: python
+        Example::
 
             >>> order_by = OrderBy('name')
             >>> order_by.opposite
             '-name'
 
-        :rtype: `.OrderBy` object
-        """
+        '''
         return OrderBy(self[1:]) if self.is_descending else OrderBy('-' + self)
 
     @property
     def is_descending(self):
-        """
-        Return `True` if this object induces *descending* ordering
-
-        :rtype: `bool`
-        """
+        '''
+        Returns `True` if this object induces *descending* ordering.
+        '''
         return self.startswith('-')
 
     @property
     def is_ascending(self):
-        """
-        Return `True` if this object induces *ascending* ordering.
-
-        :returns: `bool`
-        """
+        '''
+        Returns `True` if this object induces *ascending* ordering.
+        '''
         return not self.is_descending
 
+    def for_queryset(self):
+        '''
+        Returns the current instance usable in Django QuerySet's order_by
+        arguments.
+        '''
+        return self.replace(Accessor.SEPARATOR, OrderBy.QUERYSET_SEPARATOR)
+
 
 @six.python_2_unicode_compatible
 class OrderByTuple(tuple):
-    """
-    Stores ordering as (as `.OrderBy` objects). The
-    `~django_tables2.tables.Table.order_by` property is always converted
-    to an `.OrderByTuple` object.
+    '''
+    Stores ordering as (as `.OrderBy` objects). The `~.Table.order_by` property
+    is always converted to an `.OrderByTuple` object.
 
     This class is essentially just a `tuple` with some useful extras.
 
-    Example:
-
-    .. code-block:: python
+    Example::
 
         >>> x = OrderByTuple(('name', '-age'))
         >>> x['age']
@@ -127,7 +137,7 @@ class OrderByTuple(tuple):
         >>> x['age'].opposite
         'age'
 
-    """
+    '''
     def __new__(cls, iterable):
         transformed = []
         for item in iterable:
@@ -140,23 +150,23 @@ class OrderByTuple(tuple):
         return ','.join(self)
 
     def __contains__(self, name):
-        """
+        '''
         Determine if a column has an influence on ordering.
 
-        Example:
+        Example::
 
-        .. code-block:: python
-
-            >>> ordering =
             >>> x = OrderByTuple(('name', ))
             >>> 'name' in  x
             True
             >>> '-name' in x
             True
 
-        :param name: The name of a column. (optionally prefixed)
-        :returns: `bool`
-        """
+        Arguments:
+            name (str): The name of a column. (optionally prefixed)
+
+        Returns:
+            bool: `True` if the column with `name` influences the ordering.
+        '''
         name = OrderBy(name).bare
         for order_by in self:
             if order_by.bare == name:
@@ -164,13 +174,11 @@ class OrderByTuple(tuple):
         return False
 
     def __getitem__(self, index):
-        """
+        '''
         Allows an `.OrderBy` object to be extracted via named or integer
         based indexing.
 
-        When using named based indexing, it's fine to used a prefixed named.
-
-        .. code-block:: python
+        When using named based indexing, it's fine to used a prefixed named::
 
             >>> x = OrderByTuple(('name', '-age'))
             >>> x[0]
@@ -180,8 +188,12 @@ class OrderByTuple(tuple):
             >>> x['-age']
             '-age'
 
-        :rtype: `.OrderBy` object
-        """
+        Arguments:
+            index (int): Index to query the ordering for.
+
+        Returns:
+            `.OrderBy`: for the ordering at the index.
+        '''
         if isinstance(index, six.string_types):
             for order_by in self:
                 if order_by == index or order_by.bare == index:
@@ -237,9 +249,9 @@ class OrderByTuple(tuple):
         return Comparator
 
     def get(self, key, fallback):
-        """
+        '''
         Identical to __getitem__, but supports fallback value.
-        """
+        '''
         try:
             return self[key]
         except (KeyError, IndexError):
@@ -247,37 +259,40 @@ class OrderByTuple(tuple):
 
     @property
     def opposite(self):
-        """
-        Return version with each `.OrderBy` prefix toggled.
-
-        Example:
-
-        .. code-block:: python
+        '''
+        Return version with each `.OrderBy` prefix toggled::
 
             >>> order_by = OrderByTuple(('name', '-age'))
             >>> order_by.opposite
             ('-name', 'age')
-        """
+        '''
         return type(self)((o.opposite for o in self))
 
 
 class Accessor(str):
-    """
+    '''
     A string describing a path from one object to another via attribute/index
     accesses. For convenience, the class has an alias `.A` to allow for more concise code.
 
     Relations are separated by a ``.`` character.
-    """
+    '''
     SEPARATOR = '.'
 
     def resolve(self, context, safe=True, quiet=False):
-        """
+        '''
         Return an object described by the accessor by traversing the attributes
         of *context*.
 
-        Example:
+        Lookups are attempted in the following order:
 
-        .. code-block:: python
+         - dictionary (e.g. ``obj[related]``)
+         - attribute (e.g. ``obj.related``)
+         - list-index lookup (e.g. ``obj[int(related)]``)
+
+        Callable objects are called, and their result is used, before
+        proceeding with the resolving.
+
+        Example::
 
             >>> x = Accessor('__len__')
             >>> x.resolve('brad')
@@ -286,26 +301,18 @@ class Accessor(str):
             >>> x.resolve('brad')
             'B'
 
-        :type  context: `object`
-        :param context: The root/first object to traverse.
-        :type     safe: `bool`
-        :param    safe: Don't call anything with ``alters_data = True``
-        :type    quiet: bool
-        :param   quiet: Smother all exceptions and instead return `None`
-        :returns: target object
-        :raises: anything ``getattr(a, "b")`` raises, e.g. `TypeError`,
-                 `AttributeError`, `KeyError`, `ValueError` (unless *quiet* ==
-                 `True`)
+        Arguments:
+            context (object): The root/first object to traverse.
+            safe (bool): Don't call anything with `alters_data = True`
+            quiet (bool): Smother all exceptions and instead return `None`
 
-        `~.Accessor.resolve` attempts lookups in the following order:
+        Returns:
+            target object
 
-        - dictionary (e.g. ``obj[related]``)
-        - attribute (e.g. ``obj.related``)
-        - list-index lookup (e.g. ``obj[int(related)]``)
-
-        Callable objects are called, and their result is used, before
-        proceeding with the resolving.
-        """
+        Raises:
+            TypeError`, `AttributeError`, `KeyError`, `ValueError`
+            (unless `quiet` == `True`)
+        '''
         try:
             current = context
             for bit in self.bits:
@@ -348,7 +355,9 @@ class Accessor(str):
         return self.split(self.SEPARATOR)
 
     def get_field(self, model):
-        '''Return the django model field for model in context, following relations'''
+        '''
+        Return the django model field for model in context, following relations.
+        '''
         if not hasattr(model, '_meta'):
             return
 
@@ -358,9 +367,16 @@ class Accessor(str):
                 field = model._meta.get_field(bit)
             except FieldDoesNotExist:
                 break
-            if hasattr(field, 'rel') and hasattr(field.rel, 'to'):
-                model = field.rel.to
-                continue
+
+            if hasattr(field, 'remote_field'):
+                rel = getattr(field, 'remote_field', None)
+                model = getattr(rel, 'model', model)
+
+            # !!! Support only for Django <= 1.8
+            # Remove this when support for Django 1.8 is over
+            else:
+                rel = getattr(field, 'rel', None)
+                model = getattr(rel, 'to', model)
 
         return field
 
@@ -370,8 +386,11 @@ class Accessor(str):
          - the resolved left part.
          - the remainder
 
-        >>> Accessor('a.b.c').penultimate({'a': {'a': 1, 'b': {'c': 2, 'd': 4}}})
-        ({'c': 2, 'd': 4}, 'c')
+        Example::
+
+            >>> Accessor('a.b.c').penultimate({'a': {'a': 1, 'b': {'c': 2, 'd': 4}}})
+            ({'c': 2, 'd': 4}, 'c')
+
         '''
         path, _, remainder = self.rpartition('.')
         return A(path).resolve(context, quiet=quiet), remainder
@@ -381,15 +400,22 @@ A = Accessor  # alias
 
 
 class AttributeDict(dict):
-    """
+    '''
     A wrapper around `dict` that knows how to render itself as HTML
     style tag attributes.
 
     The returned string is marked safe, so it can be used safely in a template.
     See `.as_html` for a usage example.
-    """
+    '''
+    blacklist = ('th', 'td', '_ordering')
+
+    def _iteritems(self):
+        for k, v in six.iteritems(self):
+            if k not in self.blacklist:
+                yield (k, v() if callable(v) else v)
+
     def as_html(self):
-        """
+        '''
         Render to HTML tag attributes.
 
         Example:
@@ -403,28 +429,25 @@ class AttributeDict(dict):
 
         :rtype: `~django.utils.safestring.SafeUnicode` object
 
-        """
-        return mark_safe(' '.join(['%s="%s"' % (k, escape(v if not callable(v) else v()))
-                                   for k, v in six.iteritems(self)]))
+        '''
+        return format_html_join(' ', '{}="{}"', self._iteritems())
 
 
 def segment(sequence, aliases):
-    """
+    '''
     Translates a flat sequence of items into a set of prefixed aliases.
 
     This allows the value set by `.QuerySet.order_by` to be translated into
     a list of columns that would have the same result. These are called
-    "order by aliases" which are optionally prefixed column names.
+    "order by aliases" which are optionally prefixed column names::
 
-    e.g.
+        >>> list(segment(('a', '-b', 'c'),
+        ...              {'x': ('a'),
+        ...               'y': ('b', '-c'),
+        ...               'z': ('-b', 'c')}))
+        [('x', '-y'), ('x', 'z')]
 
-        >>> list(segment(("a", "-b", "c"),
-        ...              {"x": ("a"),
-        ...               "y": ("b", "-c"),
-        ...               "z": ("-b", "c")}))
-        [("x", "-y"), ("x", "z")]
-
-    """
+    '''
     if not (sequence or aliases):
         return
     for alias, parts in aliases.items():
@@ -449,9 +472,10 @@ def segment(sequence, aliases):
 
 def signature(fn):
     '''
-    Returns a tuple containing:
-     - the arguments (positional or keyword)
-     - the name of the ** kwarg catch all.
+    Returns:
+        tuple: Returns a (arguments, kwarg_name)-tuple:
+             - the arguments (positional or keyword)
+             - the name of the ** kwarg catch all.
 
     The self-argument for methods is always removed.
     '''
@@ -462,32 +486,46 @@ def signature(fn):
     if six.PY2:
         argspec = inspect.getargspec(fn)
         args = argspec.args
-        return (
-            tuple(args[1:] if args[0] == 'self' else args),
-            argspec.keywords
-        )
-    else:
-        signature = inspect.signature(fn)
+        if len(args) > 0:
+            args = tuple(args[1:] if args[0] == 'self' else args)
 
-        args = []
-        keywords = None
-        for arg in signature.parameters.values():
-            if arg.kind == arg.VAR_KEYWORD:
-                keywords = arg.name
-            elif arg.kind == arg.VAR_POSITIONAL:
-                # skip *args catch-all
-                continue
-            else:
-                args.append(arg.name)
+        return (args, argspec.keywords)
 
-        return tuple(args), keywords
+    # python 3 version:
+    signature = inspect.signature(fn)
+
+    args = []
+    keywords = None
+    for arg in signature.parameters.values():
+        if arg.kind == arg.VAR_KEYWORD:
+            keywords = arg.name
+        elif arg.kind == arg.VAR_POSITIONAL:
+            continue  # skip *args catch-all
+        else:
+            args.append(arg.name)
+
+    return tuple(args), keywords
 
 
-def computed_values(d, *args, **kwargs):
-    """
+def call_with_appropriate(fn, kwargs):
+    '''
+    Calls the function ``fn`` with the keyword arguments from ``kwargs`` it expects
+
+    If the kwargs argument is defined, pass all arguments, else provide exactly
+    the arguments wanted.
+    '''
+    args, keyword = signature(fn)
+    if not keyword:
+        kwargs = {key: kwargs[key] for key in kwargs if key in args}
+
+    return fn(**kwargs)
+
+
+def computed_values(d, kwargs=None):
+    '''
     Returns a new `dict` that has callable values replaced with the return values.
 
-    Simple example:
+    Example::
 
         >>> compute_values({'foo': lambda: 'bar'})
         {'foo': 'bar'}
@@ -497,7 +535,7 @@ def computed_values(d, *args, **kwargs):
     1. If the value is callable, call it and make that the new value.
     2. If the value is an instance of dict, use ComputableDict to compute its keys.
 
-    Example:
+    Example::
 
         >>> def parents():
         ...     return {
@@ -513,13 +551,20 @@ def computed_values(d, *args, **kwargs):
         >>> computed_values(a)
         {'name': 'Brad', 'parents': {'father': 'Foo', 'mother': 'Bar'}}
 
-    :rtype: dict
-    """
+    Arguments:
+        d (dict): The original dictionary.
+        kwargs: any extra keyword arguments will be passed to the callables, if the callable
+            takes an argument with such a name.
+
+    Returns:
+        dict: with callable values replaced.
+    '''
+    kwargs = kwargs or {}
     result = {}
     for k, v in six.iteritems(d):
         if callable(v):
-            v = v(*args, **kwargs)
+            v = call_with_appropriate(v, kwargs=kwargs)
         if isinstance(v, dict):
-            v = computed_values(v, *args, **kwargs)
+            v = computed_values(v, kwargs=kwargs)
         result[k] = v
     return result
